/*
 * Sentinella
 * http://sentinella.sourceforge.net/
 * Copyright (c) 2009, 2010 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "PlayAlarm.h"

PlayAlarm::PlayAlarm(const QString& url) :
	QObject(), AbstractAction(), url(url), output(NULL) {

}

PlayAlarm::~PlayAlarm() {
	delete output;
}

void PlayAlarm::execute() {
	media.setPrefinishMark(200);
	media.setCurrentSource(url);

	output = new Phonon::AudioOutput(Phonon::NotificationCategory);
	Phonon::createPath(&media, output);

	connect(&media, SIGNAL(aboutToFinish()), this, SLOT(enqueue()));
	media.play();
}

void PlayAlarm::stop() {
	disconnect(&media, SIGNAL(aboutToFinish()), this, SLOT(enqueue()));
	media.stop();
}

void PlayAlarm::enqueue() {
	media.enqueue(url);
}

#include "PlayAlarm.moc"
