/*
 * Sentinella
 * http://sentinella.sourceforge.net/
 * Copyright (c) 2011 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef PROCESS_H
#define PROCESS_H

#include <libsysactivity/libsysactivity.h>
#include <sys/types.h>
#include "common.h"

class Process {
public:
	Process();
	~Process();
	bool processExists(const pid_t pid);
	void getProcess(struct sa_process* dst) _NONNULL;
	int error;
};

#endif /* PROCESS_H */
